﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                           module_demonology.lua
--
--	Copyright (C) 2012 Maighurl
--
--  This file is for use with clcInfo by abija.  Some of the code was pulled
--  from existing clcInfo modules and the example module.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      Maighurl
------------------------------------------------------------------------------
-- Description:
--   Actions module for demonology warlocks
------------------------------------------------------------------------------
if not clcInfo then return end       -- need clcInfo to run
if not clcInfo_Warlock then return end -- need clcInfo_Warlock
local _, englishClass = UnitClass("player") -- need to be a Warlock
if englishClass ~= clcInfo_Warlock.CLASS then return end

print("|c009482C9clcInfo_Warlock:Demonology module loaded|r")

-- Get local copies of functions/tables.  This will speed up execution.
local clcInfo_Warlock = clcInfo_Warlock
local spells = clcInfo_Warlock.spells
local GetTimeLeft = clcInfo_Warlock.GetTimeLeft
local GetCooldown = clcInfo_Warlock.GetCooldown
local IconNext = clcInfo_Warlock.IconNext
local IconCurrent = clcInfo_Warlock.IconCurrent
local CLIP_HARMFUL = clcInfo_Warlock.CLIP_HARMFUL
local UnitPower = UnitPower


local is_in_flight_immolate = clcInfo_Warlock.spells["Immolate"].IsInFlight
local is_in_flight_soul_fire = clcInfo_Warlock.spells["Soul Fire"].IsInFlight
local is_in_flight_hand_of_guldan = clcInfo_Warlock.spells["Hand of Guldan"].IsInFlight
local is_in_flight_shadowbolt = clcInfo_Warlock.spells["Shadow Bolt"].IsInFlight

local guess_travel_time_immolate = clcInfo_Warlock.spells["Immolate"].GuessTravelTime
local guess_travel_time_hand_of_guldan = clcInfo_Warlock.spells["Hand of Guldan"].GuessTravelTime

local get_time_left_demon_soul = clcInfo_Warlock.GetTimeLeftDemonSoul


-- Assign module variables
local specname = "demonology"
local prio_arg = "demonologyprio"
local sample_prio = "bod, ds_fg, ds_fh, immolate, corruption, shadowflame"
local priority = {"meta", "ds_fg", "immolate", "hog", "ds_fh", "bod", "immoAura", "corruption", "shadowflame", "incinerate", "st_sb", "sb", "soulfire" }
local class_module
local class_module_name

local db_default = {
    char_var = false,
}
local db

local tdb_default = {
    version = clcInfo_Warlock.VERSION,
    priority = priority,
}
local tdb

-- actions ---------------------------------------------------------------------
-- local copies of spell names and ids to ugly up code but speed action
-- functions up
-- we want the action functions to run as quickly as possible


local BaneofAgony = spells["Bane of Agony"].name
local BaneofDoom = spells["Bane of Doom"].name
local Corruption = spells["Corruption"].name
local Decimation = spells["Decimation"].name
local DemonSoul = spells["Demon Soul"].name
local HandofGuldan = spells["Hand of Guldan"].name
local Immolate = spells["Immolate"].name
local ImmolationAura = spells["Immolation Aura"].name
local Incinerate = spells["Incinerate"].name
local Metamorphosis = spells["Metamorphosis"].name
local MoltenCore = spells["Molten Core"].name
local ShadowBolt = spells["Shadow Bolt"].name
local SoulFire = spells["Soul Fire"].name
local ShadowBolt = spells["Shadow Bolt"].name
local ShadowTrance = spells["Shadow Trance"].name

local BaneofDoomId = spells["Bane of Doom"].id
local ChaosBoltId = spells["Chaos Bolt"].id
local CorruptionId = spells["Corruption"].id
local DemonSoulId = spells["Demon Soul"].id
local HandofGuldanId = spells["Hand of Guldan"].id
local ImmolateId = spells["Immolate"].id
local ImmolationAuraId = spells["Immolation Aura"].id
local IncinerateId = spells["Incinerate"].id
local MetamorphosisId = spells["Metamorphosis"].id
local ShadowburnId = spells["Shadowburn"].id
local ShadowflameId = spells["Shadowflame"].id
local SoulburnId = spells["Soulburn"].id
local SoulFireId = spells["Soul Fire"].id
local ShadowBoltId = spells["Shadow Bolt"].id

-- Demonology Priority List as implemented in this module
--  1) Metamorphosis
--  2) Demon Soul:Felguard
--  3) Immolate
--  4) Hand of Gul'dan
--  5) Demon Soul:Felhunter
--  6) Bane of Doom
--  7) Immolation Aura
--  8) Corruption
--  9) Shadowflame
-- 10) Shadow Bolt:Shadow Trance
-- 11) Incinerate:Molten Core
-- 12) Soul Fire:Decimation
-- 13) Shadow Bolt

local actions = {
    -- Demon Soul with Felhunter
    ds_fh = {
        id = DemonSoulId,
        GetCD = function(_, cur_id, mana)
            if cur_id ~= DemonSoulId -- can't cast this twice in a row
               and UnitExists("playerpet") and not IsMounted() -- can't cast without a pet
               and strfind(UnitCreatureFamily("playerpet"), "Felhunter") then
                local _, _, _, cost = GetSpellInfo(DemonSoulId)
                if (mana > cost or cur_id == LifeTapId) then
                     -- can't cast without mana
                    return GetCooldown(DemonSoulId), 0
                end
            end
            return 100
        end,
        info = spells["Demon Soul:Felhunter"].name,
    },

    -- Demon Soul with Felguard
    ds_fg = {
        id = DemonSoulId,
        GetCD = function(_, cur_id, mana)
            if cur_id ~= DemonSoulId -- can't cast this twice in a row
               and UnitExists("playerpet") and not IsMounted() -- can't cast without a pet
               and strfind(UnitCreatureFamily("playerpet"), "Felguard") then 
                local _, _, _, cost = GetSpellInfo(DemonSoulId)
                if (mana > cost or cur_id == LifeTapId) then
                     -- can't cast without mana
                    return GetCooldown(DemonSoulId), 0
                end
            end
            return 100
        end,
        info = spells["Demon Soul:Felguard"].name,
    },

    -- Immolate
    immolate = {
		id = ImmolateId,
		GetCD = function(casting_name, cur_id, mana)
            -- don't cast if already casting or one is on the way
            if cur_id ~= ImmolateId
               and casting_name ~= Immolate
               and not is_in_flight_immolate() then
                local _, _, _, cost, _, _, casting_time = GetSpellInfo(ImmolateId)
                if (mana > cost or cur_id == LifeTapId) then
                    -- Do not cast until the time left on the DoT is low enough.
                    -- Return the time left MINUS the casting time + travel time
                    -- and then subtract the clip time as well.  Return it or the
                    -- spell CD, whichever is greater.
                    return max(GetTimeLeft("target", Immolate, "HARMFUL|PLAYER")
                                - (casting_time/1000 + guess_travel_time_immolate())
                                - CLIP_HARMFUL, 0), 1
                end
            end
            return 100
		end,
		info = spells["Immolate"].name,
	},
	
	-- Immolation Aura
    immoAura = {
	  id = ImmolationAuraId,
	  GetCD = function(casting_name, cur_id, mana)
	      -- must be in demon form
        if GetTimeLeft("player", Metamorphosis, "HELPFUL") > 0 then
            return GetCooldown(ImmolationAuraId), 1
        end
	      return 100
	  end,
	  info = spells["Immolation Aura"].name,
    },
	
	-- Metamorphosis
    meta = {
	      id = MetamorphosisId,
	      GetCD = function(casting_name, cur_id, mana)
	          if cur_id ~= MetamorphosisId then
	              return GetCooldown(MetamorphosisId), 1
            end
            return 100
	      end,
	      info = spells["Metamorphosis"].name,
    },

    -- Hand of Gul'dan	
    hog = {
    id = HandofGuldanId,
    GetCD = function(casting_name, cur_id, mana)
        if cur_id ~= HandofGuldanId and casting_name ~= HandofGuldan then
	          local _, _, _, cost = GetSpellInfo(HandofGuldanId)
            if (mana > cost or cur_id == LifeTapId) then
 	              return GetCooldown(HandofGuldanId), 1
 	          end
	      end
	      return 100
  	end,
  	info = spells["Hand of Guldan"].name,
  	},  

    -- Shadow Bolt:Shadow Trance
    sb_st = {
    id = ShadowBoltId,
    GetCD = function(_, cur_id, mana)
	      if not is_in_flight_shadowbolt() then
            if GetTimeLeft("player", ShadowTrance, "HELPFUL") > 0 then
	              local _, _, _, cost = GetSpellInfo(ShadowBoltId)
                if (mana > cost or cur_id == LifeTapId) then
	    	            return GetCooldown(ShadowBoltId), 1
	    	        end
	    	    end
	      end
	      return 100
  	end,
  	info = spells["Shadow Bolt"].name..":"..spells["Shadow Trance"].name,
  	},  

    -- Shadow Bolt
    sb = {
    id = ShadowBoltId,
    GetCD = function(_, cur_id, mana)
	      if not is_in_flight_shadowbolt() then
	          local _, _, _, cost = GetSpellInfo(ShadowBoltId)
            if (mana > cost or cur_id == LifeTapId) then
                return GetCooldown(ShadowBoltId), 1
	    	    end
	      end
	      return 100
  	end,
  	info = spells["Shadow Bolt"].name,
  	},  

    -- Incinerate:Molten Core
    incinerate = {
    id = IncinerateId,
		GetCD = function(casting_name, cur_id, mana)
        -- CAN be suggested twice in a row, so don't restrict
        -- just check mana
        if GetTimeLeft("player", MoltenCore, "HELPFUL") > 0 then
            local _, _, _, cost = GetSpellInfo(IncinerateId)
            if (mana > cost or cur_id == LifeTapId) then
                return GetCooldown(IncinerateId), 1
            end
        end
        return 100
    end,
		info = spells["Incinerate"].name..":"..spells["Molten Core"].name,
    },

    -- Bane of Doom
    bod = {
		id = BaneofDoomId,
		GetCD = function(_, cur_id, mana)
            if cur_id ~= BaneofDoomId then
                local _, _, _, cost = GetSpellInfo(BaneofDoomId)
                if (mana > cost or cur_id == LifeTapId) then
                    return max(GetTimeLeft("target", BaneofDoom, "HARMFUL|PLAYER")-CLIP_HARMFUL,
                                 GetTimeLeft("target", BaneofAgony, "HARMFUL|PLAYER")), 1
                end
            end
            return 100
		end,
		info = spells["Bane of Doom"].name,
	  },
	  
	  -- Corruption
    corruption = {
		id = CorruptionId,
		GetCD = function(_, cur_id, mana)
            if cur_id ~= CorruptionId then
                local _, _, _, cost = GetSpellInfo(CorruptionId)
                if (mana > cost or cur_id == LifeTapId) then
                    return max(GetTimeLeft("target", Corruption, "HARMFUL|PLAYER")-CLIP_HARMFUL,0), 1
                end
            end
            return 100
		end,
		info = spells["Corruption"].name,
	},

    -- Shadowflame
    shadowflame = {
		id = ShadowflameId,
		GetCD = function(_, cur_id, mana, distance)
            -- since distance is max_distance from GetRange, we're looking
            -- for the 8 from the 5,8 pair.
            -- since it has a cd, don't suggest twice
            if cur_id ~= ShadowflameId and distance <= 8 then
                local _, _, _, cost = GetSpellInfo(ShadowflameId)
                if (mana > cost or cur_id == LifeTapId) then
                    return GetCooldown(ShadowflameId), 1
                end
            end
            return 100
		end,
		info = spells["Shadowflame"].name,
	},


    -- Soul Fire:Decimation
    soulfire = {
    id = SoulFireId,
    GetCD = function(casting_name, cur_id, mana)
        if cur_id ~= SoulFireId and casting_name ~= SoulFire and not is_in_flight_soul_fire() then
            if GetTimeLeft("player", Decimation, "HELPFUL") > 0 then
	              local _, _, _, cost = GetSpellInfo(SoulFireId)
                if (mana > cost or cur_id == LifeTapId) then
	    	            return GetCooldown(SoulFireId), 1
	    	        end
	    	    end
	      end
	      return 100
  	end,
  	info = spells["Soul Fire"].name..":"..spells["Decimation"].name,
  	},  



}

----FUNCTIONS-------------------------------------------------------------------
-- The following are called once or infrequently, so speed isn't an issue.

-- called whenever module options are loaded
local GetDB
local SetDB
local GetTDB
local SetTDB
local GetPriority
local SetPriority

--initialize module.
class_module, class_module_name = clcInfo_Warlock:init_module(specname)

--called when template updates.
function class_module:OnTemplatesUpdate()
    tdb, GetTDB, SetTDB, GetPriority, SetPriority
        = clcInfo_Warlock:OnTemplatesUpdate(class_module_name, actions,
                                            tdb_default, specname)
end

-- called each time clcInfo initializes
function class_module.OnInitialize()
    db, GetDB, SetDB = clcInfo_Warlock:OnInitialize(class_module_name,
                                                    db_default)
end

-- called when module options are opened
local function LoadDemonologyModule()
    local _, talent_tree_name = GetTalentTabInfo(2)
    clcInfo_Warlock.LoadModule(class_module_name, talent_tree_name, sample_prio,
                               actions, GetDB, SetDB, GetTDB, SetTDB,
                               GetPriority, SetPriority)
end
clcInfo.optionsCMLoaders[#(clcInfo.optionsCMLoaders) + 1] = LoadDemonologyModule

-- giving queue as command line
local function CmdDemonology(args)
    clcInfo_Warlock:CmdLine(tdb, args, actions, specname)
end

clcInfo.cmdList[prio_arg] = CmdDemonology

----ABILITY ICONS--------------------------------------------------------------
----Add these to the clcInfo.env (so they live with IconAura, etc.
local emod = clcInfo.env
function emod.IconDemonologyCurrent(...)
    return IconCurrent(actions, specname, tdb.rangePerSkill)
end
function emod.IconDemonologyNext(...)
    return IconNext()
end
